/*  cmplx.cpp -- complex functions for engsci
 *  (C) 1998, C. Bond
 *  Based on Engineering and Scientific Computations in Pascal,
 *  by Lawrence P. Huelsman.
 *
 *  Polynomial coefficient order in arrays is from high to low,
 *  which is opposite to that in the reference.
 */
#include <math.h>
#include <complex>

double Mag(complex<double> x)
{
    return sqrt(norm(x));
}
double Arg(complex<double> x)
{
    double re,im,a;
    re = real(x);
    im = imag(x);
    a = 0.0;
    if (re == 0.0) {
        if (im == 0.0) return a;
        a = 0.5 * M_PI;
        if (im < 0.0) a = -a;
        return a;
    }
    a = atan2(im,re);
    return a;
}

complex<double> ValPoly(int n,double a[],complex<double> s)
{
    int i;
    complex<double> z,val;
    val = complex<double>(a[0],0);
    for (i=1;i<=n;i++) {
        z = complex<double>(a[i],0);
        val *= s;
        val += z;
    }
    return val;
}

void DifPoly(int n, double a[],double b[])
{
    int i;
    
    for (i=n-1;i>=0;i--)
        b[i] = (n-i)* a[i];
}                      
