/*  invlap.cpp -- Inverse Laplace Transform
 *  (C) 1998, C. Bond
 *  Based on Engineering and Scientific Computations in Pascal,
 *  by Lawrence P. Huelsman
 */
#include <math.h>
#include <complex>

void InvLaplace(int nt,double tr,int n,complex<double> p[],
    complex<double> r[],double a[])
{
    double t,dt,f;
    int i,j;
    complex<double> pt,ex;

    t = 0.0;
    dt = tr / nt;
    for (i = 0;i< nt;i++) {
        f = 0.0;
        for (j=0;j<n;j++) {
            if (imag(p[j])  > -1.0e-8) {
                if (fabs(imag(p[j])) < 1.0e-8) {
/* contribution from real pole */                    
                    f += (real(r[j]) * exp(real(p[j])*t));
                }
                else {
                    pt = t * p[j];
                    ex = exp(pt);
                    ex *= r[j];
                    f += (2.0 * real(ex));
                }
            }
        }
        a[i] = f;
        t += dt;
    }
}

