/* prtctst.c -- Bairstow's complex root finder
 *
 *  (C) 1991, 1998, C. Bond
 *
 *  Finds all real and complex roots of polynomials
 *  with real coefficients.
 *
 *  Features:
 *      o Global, self-starting method,
 *      o Does not require initial estimates,
 *      o Finds all roots and quadratic factors,
 *
 *  Caveats:
 *      All polynomial root finders have problems
 *      maintaining accuracy in the presence of
 *      repeated roots.
 *      This is no exception!
 */

#include <stdlib.h>
#include <stdio.h>
#include <math.h>
#include <complex>
#include "engsci.h"
#define MAX_TERMS 21
int precision_error_flag;

void main()
{
	int i,n,order=0;
	double tmp,a[MAX_TERMS];
    complex<double> p[MAX_TERMS];

	while ((order < 2) || (order > MAX_TERMS-1)) {
		printf("Polynomial order (2-20): ");
		scanf("%d",&order);
	}
	printf("Enter coefficients, high order to low order.\n");
	for (i=0;i<=order;i++){
		printf("a(%d)=",i);
		scanf("%lf",&a[i]);
		if (i == 0) {
			tmp = a[i];
			a[i] = 1.0;
		}
		else a[i] /= tmp;
	}
	n = order;
	precision_error_flag = 0;
	Roots(a,n,p);
	printf("The quadratic factors are:\n");
	
	for (i=order;i>=2;i-=2) /* print quadratics */
		printf("t^2  %+.15lg t  %+.15lg\n",a[i-1],a[i]);
	if ((n % 2) == 1)
		printf("The linear term is: \nt  %+.15lg\n",a[1]);
	for (i=0;i<n;i++)
		cout << p[i] << endl;
}

